#!/bin/bash
VER=1.0
############################################################
#---------------[ SymsByGroup by Turranius ]---------------#
#-                                                        -#
# A script for sorting releases by group. Will make dirs   #
# based on groupname and put a symlink to the actual       #
# release in a that folder.                                #
# Works for both dated dirs (any standard), and "normal"   #
# sections. Only requires that the release and groupname   #
# is seperated by a "-" (The word after the last "-" will  #
# be considered the groupname).                            #
#-[ Setup ]------------------------------------------------#
#                                                          #
# Copy this script to where ever you like. /glftpd/bin is  #
# prefered. Make it executable (chmod 755 symsbygroup.sh). #
#                                                          #
# Edit the settings in the Settings part down below.       #
#                                                          #
# GLROOT=    The rootdir of your glftpd installation.      #
#                                                          #
# EXCLUDE=   Any folders or files you wish it to skip.     #
#            Should be in the format 'file1|file2' etc.    #
#                                                          #
# GOLIVE=    Heres the thing.. Every time this script is   #
#            executed, it will clear out the symlinks it   #
#            created earlier. Since those symlinks wont    #
#            point to an actual folder (/site rather then  #
#            /glftpd/site), it will not remove any of the  #
#            actual folders, as if you remove the symlinks #
#            from inside glftpd (dont do that :).          #
#            So, after it has run the first time, make     #
#            damn sure that the symlinks created does NOT  #
#            work if you try to enter them from shell.     #
#            Once you have verified this, set this one to  #
#            TRUE and it will wipe the symlinks before it  #
#            remakes them.                                 #
#            If you like, you can keep this on false, but  #
#            then the symlinks will remain, even if the    #
#            actual folder is deleted, leaving dead syms.  #
#                                                          #
# SECTIONS=  Quite simply: Where to look for folders, and  #
#            where to create the GroupName folder to put   #
#            the symlink inside.                           #
#            For example: /site/SVCD:/site/SVCD.By.Group   #
#            That will look for releases in /site/SVCD and #
#            if its named Some.Release-Broken-Rocking, it  #
#            will create a symlink to it in                #
#            /site/SVCD.By.Group/Rocking/Some.Releas....   #
#                                                          #
#            If you are specifying a dated dir, add a "~"  #
#            to the end of the source folder, like         #
#            /site/0DAYS~:/site/0DAYS.By.Group             #
#            and it will enter every folder it finds (part #
#            from the ones excluded), and look for         #
#            releases.                                     #
#                                                          #
#            You may enter as many as you like. Please     #
#            note though; If you have 2 or more SECTIONS   #
#            that is pointing to the same destination, add #
#            those after one another. Otherwise it wont    #
#            detect that it has already wiped that         #
#            destination folder, and will wipe it again,   #
#            even if it just created those symlinks.       #
#            See the example.                              #
#                                                          #
# FORCE=     If a group has a - in its groupname, like     #
#            TEG-TV, then it will think TV is the          #
#            groupname and create that folder. With this,  #
#            you can force those things into the correct   #
#            folder by adding TV:TEG-TV                    #
#            See the example.                              #
#                                                          #
# DELAY=     You may run this script with the parameter    #
#            "test", and it will only show you what it     #
#            would have done ( symsbygroup.sh test ).      #
#            Sadly, you will probably not be able to pick  #
#            up what its doing without a delay.            #
#            Set this to how many seconds between each     #
#            release it should wait. Its only used when    #
#            running in test mode. Set to "" for no delay. #
#                                                          #
# Once you have it working like you want, crontab it to    #
# run like every 24h or something. It can take a while to  #
# run, depending on the number of sections you set it up   #
# to check in ( Dated dirs = zzz ).                        #
# Something like this to run it at 03:15AM every day;      #
#   15 3 * * * /glftpd/bin/symsbygroup.sh >/dev/null 2>&1  #
#-[ Final Word ]-------------------------------------------#
#                                                          #
# I cannot stress this enough. When GOLIVE is TRUE, it     #
# will rm -rf (wipe everything) in the destination folders #
# so do NOT use a destination folder in which you have     #
# something else. Also, after it has been executed once,   #
# be sure the symlinks created does NOT work from shell.   #
# If they do work from shell, it will wipe out every       #
# section you set it up to check from.                     #
# With a normal installation, it will not happen, but ppl  #
# have the weirdest installations out there.               #
# (Do not set GLROOT to "" and enter /glftpd/site.. in the #
# sections. NEVER use /glftpd in the sections.)            #
# There. You have been warned :)                           #
#-[ Contact ]----------------------------------------------#
#                                                          #
# Contact Turranius/Turran on Efnet. Usually in #glftpd    #
# WEB: http://www.grandis.nu & http://grandis.mine.nu      #
# EMAIL: turranius@hotmail.com                             #
#-[Settings ]----------------------------------------------#
#

GLROOT=/glftpd
EXCLUDE='GROUPS|!Today|!Yesterday|.message'

GOLIVE="FALSE"

SECTIONS="
/site/SVCD:/site/Syms/Movies.By.Group
/site/DIVX:/site/Syms/Movies.By.Group
/site/0DAYS~:/site/Syms/0DAYS.By.Group
/site/MP3~:/site/Syms/MP3.By.Group
"

FORCE="
VCD:TEG-VCD
TV:TEG-TV
KINESIS:KiNESiS
PTiNTERNAL:PTi
PTi_iNTERNAL:PTi
Myth:MYTH
" # Replace what with what?

DELAY="0.8"


############################################################
# No changes above here... HA, GOTCHA. I ment below.       #
############################################################

for section in $SECTIONS; do
  source="$( echo $section | awk -F":" '{print $1}' )"
  destin="$( echo $section | awk -F":" '{print $2}' )"
  dated="$( echo $section | awk -F"~" '{print $2}' )"
 
  if [ "$GOLIVE" = "TRUE" ]; then
    if [ "$LAST" != "$destin" ]; then
      if [ "$1" = "test" ]; then
        echo "Since GOLIVE is TRUE, I would have delled contents of $GLROOT$destin"
      else
        rm -rf $GLROOT$destin/*
      fi
      LAST="$destin"
    fi
  fi

  if [ "$1" != "test" ]; then
    if [ -e $GLROOT$destin/_Being.Updated.Please.Wait_ ]; then
      ok=yes
    else
      mkdir $GLROOT$destin/_Being.Updated.Please.Wait_
    fi
  fi

  DATEDFOLDER="FALSE"
  if [ "$dated" != "" ]; then
    DATEDFOLDER="TRUE"
    source="$( echo $section | awk -F"~" '{print $1}' )"
    if [ "$1" = "test" ]; then
      echo "$source is a DATED folder. Will go into each folder in it."
    fi
  fi

  if [ "$source" = "" -o "$destin" = "" ]; then
    echo "Cant read source and/or destination from $section"
    echo "Source=$source"
    echo "Destination=$destin"
    exit 0
  fi

  if [ -e "$GLROOT$source" ]; then
    sourceok="yes"
  else
    echo "Can not find the source folder $GLROOT$source from $section. Check setup"
    exit 0
  fi
  
  if [ -e "$GLROOT$destin" ]; then
    destinok="yes"
  else
    echo "Can not find destination folder $GLROOT$destin from $section. Check setup"
    exit 0
  fi

  cd $GLROOT$source
  if [ "$DATEDFOLDER" = "TRUE" ]; then
    folders="$( ls | egrep -vi $EXCLUDE )"
    for folder in $folders; do
      cd $GLROOT$source/$folder
      for i in `ls -f -A | egrep -v $EXCLUDE`; do
        verify="$( echo $i | grep -- "-" )"
        if [ "$verify" != "" ]; then
          GROUPTEST="$( echo $i | tr -s '-' ' ' )"
          for field in $GROUPTEST; do
            GROUP="$field"
          done
 
          ## Check forced folders..
          if [ "$FORCE" != "" ]; then
            for forced in $FORCE; do
              FNAME="$( echo $forced | awk -F":" '{print $1}' )"
              if [ "$FNAME" != "" ]; then
                if [ "$FNAME" = "$GROUP" ]; then
                  FNEWNAME="$( echo $forced | awk -F":" '{print $2}' )"
                  GROUP="$FNEWNAME"
                fi
              fi
            done
          fi

         ## Create group folder if it dosnt exist.
         if [ -e "$GLROOT$destin/$GROUP" ]; then
           ok="yepp"
         else
           if [ "$1" = "test" ]; then
             ok="yepp"
           else
             mkdir $GLROOT$destin/$GROUP
           fi
         fi

         ## Make the symlinks...
         if [ "$1" = "test" ]; then
           echo "$GLROOT$source/$folder/$i > $GLROOT$destin/$GROUP/$i"
         else
           ln -s $source/$folder/$i $GLROOT$destin/$GROUP/$i >/dev/null 2>&1
         fi
       fi

       if [ "$TESTDELAY" = "TRUE" ]; then
          if [ "$1" = "test" ]; then
            sleep $DELAY
          fi
        fi
      done
    done
  else
    for i in `ls -f -A | egrep -v $EXCLUDE`; do
      verify="$( echo $i | grep -- "-" )"
      if [ "$verify" != "" ]; then
        GROUPTEST="$( echo $i | tr -s '-' ' ' )"
        for field in $GROUPTEST; do
          GROUP="$field"
        done
 
        ## Check forced folders..
        if [ "$FORCE" != "" ]; then
          for forced in $FORCE; do
            FNAME="$( echo $forced | awk -F":" '{print $1}' )"
            if [ "$FNAME" != "" ]; then
              if [ "$FNAME" = "$GROUP" ]; then
                FNEWNAME="$( echo $forced | awk -F":" '{print $2}' )"
                GROUP="$FNEWNAME"
              fi
            fi
          done
        fi

        ## Create group folder if it dosnt exist.
        if [ -e "$GLROOT$destin/$GROUP" ]; then
          ok="yepp"
        else
          if [ "$1" = "test" ]; then
            ok="yepp"
          else
            mkdir $GLROOT$destin/$GROUP
          fi
        fi

        ## Make the symlinks...
        if [ "$1" = "test" ]; then
          echo "$GLROOT$source/$i > $GLROOT$destin/$GROUP/$i"
        else
          ln -s $source/$i $GLROOT$destin/$GROUP/$i >/dev/null 2>&1
        fi
      fi
    done
    if [ "$DELAY" != "" ]; then
      if [ "$1" = "test" ]; then
        sleep $DELAY
       fi
    fi
  fi

  if [ -e "$GLROOT$destin/_Being.Updated.Please.Wait_" ]; then
    rmdir $GLROOT$destin/_Being.Updated.Please.Wait_
  fi
done